%projectile
%
% Projectile simulator. Ignores air resistance! User inputs initial
% elevation, velocity, elevation step angle size and velocity step size,
% and strength of gravity and initial height.
% A parabola is then drawn. Elevation and velocity are modified by pressing
% the arrow keys. Left/Right controls the velocity, up/down controls the
% elevation.
%
% LAST UPDATED by Andy French May 2016

function projectile

%Initialise global data store d which all sub-functions can read and
%write to
global d

%Define the number of data points in trajectory
d.N = 1000;

%Set font size of axes annotation
d.fsize = 16;

%Set step size for variable parameters
d.delev_deg = 1;
d.dh = 0.1;
d.du = 0.5;

%Significant figures for title numbers
d.sfs=3;

%

%% Command line interface %%

% Display welcome message to command line
disp(' ');
disp(' Welcome to PROJECTILE ! ' )
disp(' By Andy French May 2016' )
disp(' ')
disp(' Action keys:')
disp(' q             Quit ')
disp(' Up arrow      Increase elevation')
disp(' Down arrow    Decrease elevation')
disp(' Left arrow    Decrease velocity')
disp(' Right arrow   Increase velocity')
disp(' a             Increase launch height')
disp(' z             Decrease launch height')
disp(' p             Print screen')
disp(' r             Reset')
disp(' ')

%Special test input!
default = input(' Use defaults? ','s' );
if ( strcmp( default, 'Y' )==1 ) || ( strcmp( default, 'y' )==1 )
    disp(' ')
    disp( ' Default option selected ...' );
    d.elev_deg = 45;
    d.u = 10;
    d.g = 9.81;
    d.h = 1;
else
    
    %Request initial elevation /degrees
    disp(' ')
    disp(' Request initial elevation /degrees')
    d.elev_deg = str2num( input( ' elev_deg = ','s' ));
    
    %Request launch velocity /ms^-1
    disp(' ')
    disp(' Request launch velocity /ms^-1')
    d.u = str2num( input( ' u = ','s' ));
    
    %Request strength of gravity
    disp(' ')
    disp(' Request strength of gravity')
    d.g = str2num( input( ' g = ','s' ));
    
    %Request initial height in metres
    disp(' ')
    disp(' Request initial height in metres')
    d.h = str2num( input( ' h = ','s' ));
    
end

%Store original parameters
d.elev_deg_orig  = d.elev_deg;
d.h_orig = d.h;
d.u_orig = d.u;

%

%% Generate original trajectory %%

%Create figure and axes
d.fig = figure('name','projectile',...
    'renderer','opengl','color',[1,1,1],'InvertHardCopy', 'off','KeyPressFcn',@keypressfunc);
d.ax = axes;

%Generate default trajectory
generate_trajectory;

%Plot the trajectory and bounding parabola
d.p = plot( d.x,d.y, 'linewidth', 2 );
hold on;
d.pbound = plot( d.x,d.ybound, 'r--','linewidth', 2 );
grid on
xlabel('x /m','fontsize',d.fsize )
ylabel('y /m','fontsize',d.fsize)
box on

%Set the font size of axis annotation
set( d.ax, 'fontsize', d.fsize );

%Set axis limits
xlim( [0,d.xmax] );
ylim( [0,d.ymax] );

%Construct a string for the graph title
row1 = ['Projectile trajectory: h = ',num2str(d.h,d.sfs),'m, u = ',...
    num2str(d.u,d.sfs),'ms^{-1}, elev = ',num2str(d.elev_deg,d.sfs),'^o'];
row2 = ['R = ',num2str(d.R,d.sfs),'m, y_{max} = ',num2str(d.hmax,d.sfs),...
    'm, flight time = ',num2str(d.t(end),d.sfs),'s'];
row3 = ['impact velocity = ',num2str(d.vimpact,d.sfs),'ms^{-1}, impact angle = ',...
    num2str(d.impact_angle,d.sfs),'^o' ];
d.title = title( {row1,row2,row3}, 'fontsize',d.fsize);

%%

%Generate projectile trajectory
function generate_trajectory

%Allow read-write usage of global variable d
global d

%Convert elevation into radians
d.elev = d.elev_deg*pi/180;

%Determine the bounding parabola for the given launch velocity. Use this to
%fix the axis scales
d.ymax = d.h + (d.u^2)/(2*d.g);
d.xmax = sqrt( ( 2*(d.u^2)/d.g )*( d.h + (d.u^2)/(2*d.g) ) );

%Get the parameters of the projectile parabola y = x^2
d.a = -0.5 * ( d.g/(d.u^2) )*( 1 + tan(d.elev)^2 );
d.b = tan(d.elev);
d.c = d.h;

%Work out the range R, i.e. where the projectile impacts
d.R = (-d.b - sqrt( d.b^2 -4*d.a*d.c ) )/(2*d.a);

%Generate x,y coordinates of trajectory and bounding parabola
d.x = linspace(0,d.xmax,d.N);
d.y = d.c + d.b*d.x + d.a*d.x.^2;
d.ybound = d.h+ (d.u^2)/(2*d.g) - (d.g).*(d.x.^2)/(2*d.u^2);

%Work out the height at apogee /m
d.x_apogee = ( d.u^2 / (2*d.g) )*sin( 2*d.elev );
d.hmax = d.c + d.b*d.x_apogee + d.a*d.x_apogee.^2;

%Generate x velocities
d.vx = d.u*cos(d.elev)*ones(1,d.N);

%Work out projectile time since launch /s
d.t = d.x./d.vx;

%Compute y velocity and speed v
d.vy = d.u*sin(d.elev) - d.g*d.t;
d.v = sqrt( d.vx.^2 + d.vy.^2 );

%Work out impact velocity
d.vimpact = d.v(d.N);

%Work out impact angle /degrees form horizontal
d.impact_angle = (180/pi)*atan2( -d.vy(d.N),d.vx(d.N) );

%%

%Key press handler
function keypressfunc( fig,evnt )

%Allow read-write usage of global variable d
global d

%Get the string of the current key press
s = get( d.fig,'currentkey');

%Actions, depending on the key pressed...

if strcmp(s,'q') == 1
    %Quit
    close(d.fig)
else
    if strcmp(s, 'p') == 1
        %Print screen
        print(fig,['Projectile ',strrep(datestr(now),':','-')],'-djpeg','-r300');
        
    elseif strcmp(s, 'r') == 1
        %Default parameters
        d.elev_deg = d.elev_deg_orig;
        d.u = d.u_orig;
        d.h = d.h_orig;
        
    elseif strcmp(s, 'uparrow') == 1
        %Increase elevation
        d.elev_deg = d.elev_deg + d.delev_deg;
    elseif strcmp(s, 'downarrow') == 1
        %Decrease elevation
        d.elev_deg = d.elev_deg - d.delev_deg;
    elseif strcmp(s, 'rightarrow') == 1
        %Increase velocity
        d.u = d.u + d.du;
    elseif strcmp(s, 'leftarrow') == 1
        %Decrease velocity
        d.u = d.u - d.du;
    elseif strcmp(s, 'a') == 1
        %Increase initial height
        d.h = d.h + d.dh;
    elseif strcmp(s, 'z') == 1
        %Decrease initial height
        d.h = d.h - d.dh;
    end
    
    %Update trajectory
    generate_trajectory;
    set( d.p, 'xdata', d.x, 'ydata', d.y );
    set( d.pbound, 'xdata', d.x, 'ydata', d.ybound );
    
    %Set axis limits
    xlim( [0,d.xmax] );
    ylim( [0,d.ymax] );
    
    %Construct a string for the graph title
    row1 = ['Projectile trajectory: h = ',num2str(d.h,d.sfs),'m, u = ',...
        num2str(d.u,d.sfs),'ms^{-1}, elev = ',num2str(d.elev_deg,d.sfs),'^o'];
    row2 = ['R = ',num2str(d.R,d.sfs),'m, y_{max} = ',num2str(d.hmax,d.sfs),...
        'm, flight time = ',num2str(d.t(end),d.sfs),'s'];
    row3 = ['impact velocity = ',num2str(d.vimpact,d.sfs),'ms^{-1}, impact angle = ',...
        num2str(d.impact_angle,d.sfs),'^o' ];
    set( d.title, 'string', {row1,row2,row3} );
end

%End of code